#ifndef CAMERA_H
#define CAMERA_H

#include "RotMatrix.h"
#include "Vector.h"

/**
 * Tracks the rotation and the origin of the camera.  This is the player's
 * point of view into the game world.
 */
class Camera {
   private:

      RotMatrix rotation;
      Vector origin;

      // current camera rotation in degrees
      Float yaw, pitch;

   public:

      Camera();

      /** Sets the camera rotation matrix. */
      void setRotation(const RotMatrix &newRotation) {
	 rotation = newRotation;
      }

      /** Returns the camera rotation matrix. */
      const RotMatrix &getRotation() const {
	 return rotation;
      }

      /** sets the camera origin */
      void setOrigin(const Vector &newOrigin) {
	 origin = newOrigin;
      }

      /** returns the camera origin */
      const Vector &getOrigin() const {
	 return origin;
      }

      /** rotates the camera yaw & pitch degrees */
      void rotate(Float yaw, Float pitch);

      /** moves the camera origin by the given vector. */
      void move(const Vector &offset) {
	 origin += offset;
      }

};

#endif
