#ifndef MATERIAL_H
#define MATERIAL_H

#include <spug/RCBase.h>
#include <spug/RCPtr.h>

class Vector;

/**
 * Base class for classes implementing "face painting" algorithms.  These are
 * used in the following manner: the makeCurrent() method will be called prior
 * to rendering a surface, the setCoord() method will be called prior to
 * adding a vertex for that surface.
 */
class Material : public spug::RCBase {

   public:
      /**
       * Make the material the current material for the rendering context.
       */
      virtual void makeCurrent() = 0;

      /**
       * Maps the vertex to be rendered onto the rendering system.
       */
      virtual void setCoord(const Vector &point) = 0;
};

SPUG_RCPTR(Material);

#endif
