#ifndef MODEL_H
#define MODEL_H

#include <vector>
#include <spug/RCPtr.h>
#include <spug/Iterator.h>
#include "PolyObj.h"
#include "AxisBox.h"

/**
   A model is simply a collection of polygons.
 */
class Model : public RenderableObject {

   typedef std::vector<PolyObjPtr> PolyVec;
   PolyVec polys;

   public:

      typedef spug::Iterator<const PolyObjPtr> PolyIter;

      Model();

      /** Adds a new polygon to the model. */
      void addPoly(const PolyObjPtr &poly);

      virtual void render(Scene &scene);

      /** Returns an iterator over all of the polygons. */
      PolyIter getPolyIterator() const;

      /** 
       * returns the bounding box for the model - this is a box which
       * completely encloses the model.
       */ 
      AxisBox getBoundingBox() const;
};

SPUG_RCPTR(Model);

#endif
