
#include "MouseHandler.h"

#include <gdk/gdkx.h>
#include <iostream>
#include "GameEngine.h"
#include "DisplayDriver.h"

using namespace std;

MouseHandler::MouseHandler(GameEngine &engine) :
   driver(0),
   orgX(-1),
   orgY(-1),
   engine(engine) {
}

void MouseHandler::setWindowSize(int width, int height) {
   orgX = width / 2;
   orgY = height / 2;
}

void MouseHandler::centerMouse() {
   // warp the pointer back to the origin
   driver->warpMouse(orgX, orgY);
}

void MouseHandler::handleMouseMove(int x, int y) {
   if (!driver)
      return;

   // check the deltas, if nothing has changed ignore this.
   int dx = x - orgX, dy = y - orgY;
   if (!dx && !dy)
      return;

   handleMouseDelta(dx, dy);
   centerMouse();
}

void MouseHandler::activate(DisplayDriver *activeDriver) {

   // bind to the driver
   driver = activeDriver;

   // set size from the driver
   DisplayDriver::Size size = driver->getSize();
   setWindowSize(size.width, size.height);

   // grab the mouse
   driver->grabMouse();
   centerMouse();
}

void MouseHandler::deactivate() {

   if (driver) {
      // ungrab the pointer
      driver->grabMouse(false);

      // restore unassociated state
      driver = 0;
      orgX = orgY = -1;
   }
}

void MouseHandler::handleMouseDelta(int dx, int dy) {
   // pass the rotation on to the game engine.
   engine.getCamera().rotate(dx, dy);
}

