
#ifndef MOUSEHANDLER_H
#define MOUSEHANDLER_H

#include <gtk/gtk.h> // for GtkWidget
#include <X11/Xlib.h> // for Window & Display

class GameEngine;
class DisplayDriver;

class MouseHandler {
   private:

      // origin X and Y values - these are set to the center of the target
      // window by setWindowSize() and activate(), they are initialized to -1
      int orgX, orgY;
      DisplayDriver *driver;

      // the associated game engine
      GameEngine &engine;

      /** homes the mouse pointer to the center of the opengl panel */
      void centerMouse();

   public:
      MouseHandler(GameEngine &engine);

      /**
       * Resets the origin based on the given window size.  Normally this
       * should be called by the resize handler to indicate a new window size.
       */
      void setWindowSize(int width, int height);

      /** 
       * Should be called from the mouse movement handler - handles
       * processing for a mouse move.
       *
       * Does nothing if the handler has not been activate()ed.
       */
      void handleMouseMove(int x, int y);

      void handleMouseDelta(int dx, int dy);

      /**
       * Associates the handler with the given driver, does a "grab" and
       * causes the handle methods to start processing input.
       */
      void activate(DisplayDriver *driver);

      /**
       * Turns off the handler.  Disassociates it from the current driver,
       * ungrabs the mouse and renders the "handle" methods inert.
       */
      void deactivate();

      /**
       * Returns true if the handler is active.
       */
      bool isActive() {
	 return driver ? true : false;
      }
};

#endif

