
#include "Moveable.h"

Moveable::Moveable() : position(0, 0, 0) {
   orientation.makeYaw(0);
}

Moveable::Moveable(ModelPtr model, const Vector &position, 
		   const RotMatrix &orientation
	           ) :
   model(model),
   position(position),
   orientation(orientation) {
}

void Moveable::render(Scene &scene) {

   // if there's no model, quit now.
   if (!model)
      return;

   // store the current position/orientation
   glPushMatrix();

   // translate the rotation matrix 
   Float rotMatrix[4][4];;
   orientation.getGLMatrix(rotMatrix);

   // translate and rotate the prior to rendering
   glLoadIdentity();
   glTranslate(position.x, position.y, position.z);
   glMultMatrix<Float>(rotMatrix);

   // render the model
   model->render(scene);

   // restore the original position
   glPopMatrix();
}

void Moveable::processRenderFrame(GameEngine &engine, Float elapsed) {
   position += velocity * elapsed;
}
