
#ifndef NATIVEPERSISTOR_H
#define NATIVEPERSISTOR_H

#include <vector>
#include <spug/Exception.h>
#include <spug/RCPtr.h>
#include <spug/Tracer.h>
#include "Material.h"
#include "basictypes.h"

namespace spug { class Reader; }

class Vector;
class Color;
class Model;

void foo(spug::Reader &reader);

SPUG_RCPTR(QuadObj);

/**
   Persists and loads objects from the systems native format.

   Schemas:
      Color {
	 Float red, green, blue;
      }

      Vector {
	 Float x, y, z;
      }

      String {
	 int length;
	 char data[length];
      }

      Quad {
	 Vector vertices[4];
	 int colorIndex;
      }

      Material {
	 int type;
	 switch (type) {
	    case 0 {
	       Color color;
	    }
	    case 1 {
	       String textureName;
	       Float xppus, yppus;
	    }
      }

      Model {
	 int materialCount;
	 Material materials[materialCount];
	 int polyCount;
	 Quad polys[polyCount];
      }


 */
class NativePersistor {

   public:
      // OutOfDataError is thrown when the reader can't provide enough data to
      // parse the entity
      SPUG_EXCEPTION(OutOfDataError);

   protected:
      typedef std::vector<MaterialPtr> MaterialVector;

      Float readFloat(spug::Reader &reader);
      int readInt(spug::Reader &reader);
      void readString(spug::Reader &reader, std::string &string);
      void readColor(spug::Reader &reader, Color &color);
      void readMaterial(spug::Reader &reader, MaterialPtr &material);
      void readVector(spug::Reader &reader, Vector &point);
      QuadObjPtr readQuad(spug::Reader &reader, const MaterialVector &palette);

   public:

      // sets detailed tracing
      static spug::Tracer::Level detailed;

      void readModel(spug::Reader &reader, Model &model);
};

#endif
