
#include "PNG.h"
#include <png.h>
#include <iostream>
#include <spug/Exception.h>

#include <GL/gl.h>  // XXX please remove

using namespace std;
using namespace spug;

struct PNG::Data {
   png_structp png;
   png_infop info, endInfo;
};

namespace {
   void handleError(png_structp png, png_const_charp message) {
      throw Exception(message);
   }

   void handleWarning(png_structp png, png_const_charp message) {
      cout << "PNG Warning: " << message << endl;
   }
}

PNG::PNG() : data(new Data), rgb8(0) {

   data->png = 0;
   data->info = data->endInfo = 0;

   data->png = png_create_read_struct(PNG_LIBPNG_VER_STRING, 
				      (png_voidp)handleError,
				      handleError, 
				      handleWarning
				      );
   if (!data->png) {
      this->~PNG();
      throw Exception("Unable to initialize PNG structure");
   }

   data->info = png_create_info_struct(data->png);
   if (!data->info) {
      this->~PNG();
      throw Exception("Unable to create PNG info structure");
   }

   data->endInfo = png_create_info_struct(data->png);
   if (!data->endInfo) {
      this->~PNG();
      throw Exception("Unable to create PNG end info structure");
   }

}

extern "C" {
   static void readFunc(png_structp png, png_bytep buffer, 
			png_size_t size
			) {
      PNG::Reader *reader = 
	 reinterpret_cast<PNG::Reader *>(png_get_io_ptr(png));
      reader->read(buffer, size);
   }
}

unsigned PNG::tex;

void PNG::read(PNG::Reader &reader) {
   // set the read function to our translation function
   png_set_read_fn(data->png, &reader, readFunc);

   // read the entire file in
   png_read_png(data->png, data->info, 0, 0);

   // get image arguments
   png_get_IHDR(data->png, data->info, 
	        reinterpret_cast<png_uint_32 *>(&width), 
	        reinterpret_cast<png_uint_32 *>(&height), 
		&depth, 
		&colorType,
	        reinterpret_cast<int *>(0), 
		reinterpret_cast<int *>(0), 
		reinterpret_cast<int *>(0)
		);
}

void *PNG::getRGB8() const {
   // if we've already calculated this, return it
   if (rgb8)
      return rgb8;

   // make sure the format is what we're expecting
   if (depth == 8 && colorType == PNG_COLOR_TYPE_RGB) {

      // allocate a contiguous RGB buffer
      rgb8 = new Byte[width * height * 3];

      // transfer the data to it
      png_bytep *rows = png_get_rows(data->png, data->info);
      for (unsigned row = 0; row < height; ++row)
	 for (unsigned col = 0; col < width; ++col) {
	    unsigned cell = row * width * 3 + col * 3;
	    rgb8[cell] = rows[row][col * 3];
	    rgb8[cell + 1] = rows[row][col * 3 + 1];
	    rgb8[cell + 2] = rows[row][col * 3 + 2];
	 }
   } else {
      throw Exception("Unexpected depth or color type");
   }

   return rgb8;
}

PNG::~PNG() {
   if (data) {
      if (data->png) {

	 // if info and endInfo are non-null, set up pointers to them so we
	 // can free them along with the read structure.
	 png_infopp infoPP = 0, endInfoPP = 0;
	 if (data->info)
	    infoPP = &data->info;
	 if (data->endInfo)
	    endInfoPP = &data->endInfo;

	 png_destroy_read_struct(&data->png, infoPP, endInfoPP);
      }

      // free up the data structure itself
      delete data;
   }

   if (rgb8) delete rgb8;
}

bool PNG::check(void *buffer, size_t size) {
   return !png_sig_cmp((png_byte *)buffer, 0, sizeof(buffer));
}

