#ifndef WURLD_PNG_H
#define WURLD_PNG_H

#include <sys/types.h>

/** 
 * Safely wraps the libpng interface - which doesn't play nice with my C++
 * code during compile.
 */
class PNG {

   private:
      // opaque internal data pointer
      struct Data;
      Data *data;

      mutable unsigned char *rgb8;

      unsigned width, height;
      int depth, colorType;

   public:

      static unsigned tex;

      /** Interface for functions passed in to read data from the file */
      class Reader {
	 public:
	    virtual void read(void *buffer, size_t size) = 0;
      };

      /** 
       * Pass this the first few bytes of a PNG file to verify that it is a
       * PNG file.  Returns true if it is.
       */
      static bool check(void *header, size_t size);

      PNG();
      ~PNG();

      /**
       * Read the PNG file into memory.
       */
      void read(Reader &reader);

      /** Returns the image width in pixels. */
      unsigned getWidth() const {
	 return width;
      }

      /** Returns the image height in pixels. */
      unsigned getHeight() const {
	 return height;
      }

      /** 
       * Returns the image data as an RGB block - each pixel is represented
       * as 1 unsigned byte of red, one of green, and one of blue stored
       * consecutively.
       */
      void *getRGB8() const;

};

#endif
