#ifndef PHYSICALOBJECT_H
#define PHYSICALOBJECT_H

#include <spug/RCPtr.h>
#include "GameObject.h"
#include "Vector.h"

/**
 * Implements basic physics and collision handling.  Objects that are physical
 * should derive from this.
 */
class PhysicalObject : public GameObject {

   private:

      // the object's current position
      Vector position;

      // the object's velocity described as a vector whose magnitude is the
      // distance traveled over the next second.
      Vector velocity;

      // the maximum distance that the object is allowed to travel since the
      // last engine frame
      Float maxDistance;

      // the distance from the bottom of the object to its origin.  Used to
      // determine whether the object is resting on the ground
      Float baseHeight;

   public:

      virtual void processEngineFrame(GameEngine &engine);

      virtual void processRenderFrame(GameEngine &engine, Float elapsed);

      /** 
       * Sets the maximum distance the object can travel before colliding with
       * something else.
       */
      void setMaxDistance(Float dist) {
         maxDistance = dist;
      }

      /** Returns the object position. */
      const Vector &getPosition() const {
         return position;
      }

      /** Sets the object's position */
      void setPosition(const Vector &pos) {
         position = pos;
      }

      /** Sets the object's velocity */
      void setVelocity(const Vector &vel) {
         velocity = vel;
      }

      /** Returns the object velocity vector. */
      const Vector &getVelocity() const {
         return velocity;
      }

};

SPUG_RCPTR(PhysicalObject);

#endif
