#ifndef POLYOBJ_H
#define POLYOBJ_H

#include "Vector.h"
#include "RenderableObject.h"
#include "Material.h"
#include "AxisBox.h"

/**
   Base class for arbitrary polygons.
 */
class PolyObj : public RenderableObject {

   protected:
      MaterialPtr material;

      PolyObj(MaterialPtr material) : material(material) {}

   public:
      /**
       * Returns the normal of the polygon.
       */
      virtual Vector normal() const = 0;

      /**
       * Returns true if the ray defined by origin and direction intersects
       * with the polygon.  Sets "intersection" and "distance" if it does.  If
       * it does not, the values of these variables is undefined.
       */
      virtual bool intersects(const Vector &origin, const Vector &direction,
			      Vector &intersection,
			      Float &distance
	                      ) = 0;

      /**
       * Returns true if the ray defined by origin and direction intersects
       * with the polygon.
       */
      bool intersects(const Vector &origin, const Vector &direction) {
	 Vector intersection;
	 Float distance;
	 return intersects(origin, direction, intersection, distance);
      }

      /**
       * Returns a bounding box that can accomodate the polygon.
       */
      virtual AxisBox getBoundingBox() const = 0;

};

typedef spug::RCPtr<PolyObj> PolyObjPtr;

#endif
