
#include "PythonPlugin.h"

// prototype for getDisplayDriver
#include "PythonPluginInternal.h"
#include "PythonObject.h"
#include "Vector.h"
#include "Texture.h"
#include "TextureMaterial.h"
#include "QuadObj.h"
#include "Model.h"
#include "Moveable.h"
#include "Graphics.h"

#include <Python.h>

DisplayDriver *PythonPlugin::displayDriver;
DisplayDriver *getDisplayDriver() {
   return PythonPlugin::displayDriver;
}

void PythonPlugin::setDisplayDriver(DisplayDriver *driver) {
   displayDriver = driver;
}

GameEngine *PythonPlugin::gameEngine;
GameEngine *getGameEngine() {
   return PythonPlugin::gameEngine;
}

RotMatrix createYawRotMatrix(Float degrees) {
   RotMatrix rot;
   rot.makeYaw(degrees);
   return rot;
}

RotMatrix createPitchRotMatrix(Float degrees) {
   RotMatrix rot;
   rot.makePitch(degrees);
   return rot;
}

RotMatrix createRollRotMatrix(Float degrees) {
   RotMatrix rot;
   rot.makeRoll(degrees);
   return rot;
}

void PythonPlugin::setGameEngine(GameEngine *engine) {
   gameEngine = engine;
}

Vector createVector(Float x, Float y, Float z) {
   return Vector(x, y, z);
}

TexturePtr createTexture(const char *filename) {
   return new Texture(filename);
}

MaterialPtr createTextureMaterial(const TexturePtr &texture, 
                                  Float xppus, 
				  Float yppus
				  ) {
   return new TextureMaterial(texture, xppus, yppus);
}

PolyObjPtr createQuad(Vector c0, Vector c1, Vector c2, Vector c3,
		      const MaterialPtr &material
		      ) {
   return new QuadObj(c0, c1, c2, c3, material);
}

ModelPtr createModel() {
   return new Model();
}

MoveablePtr createMoveable() {
   return new Moveable();
}

PythonObject *createPythonObject() {
   return new PythonObject();
}

Graphics *getGraphics() {
   return &Graphics::getInstance();
}

extern "C" void initwurld();

void PythonPlugin::handleInit() {
   Py_Initialize();
   initwurld();
   FILE *fp = fopen("wurldrc.py", "r");
   if (!fp)
      throw PluginError("Unable to load startup file wurldrc.py");
   PyRun_AnyFile(fp, "wurldrc.py");
}

void PythonPlugin::handleTerminate() {
   Py_Finalize();
}
