
#include "RotMatrix.h"
#include <assert.h>

void RotMatrix::makeYaw(Float yaw) {
   yaw = rads(yaw);
   matrix[0][0] = cos(yaw);
   matrix[0][1] = 0;
   matrix[0][2] = -sin(yaw);
   matrix[1][0] = 0;
   matrix[1][1] = 1;
   matrix[1][2] = 0;
   matrix[2][0] = sin(yaw);
   matrix[2][1] = 0;
   matrix[2][2] = cos(yaw);
}

void RotMatrix::makePitch(Float pitch) {
   pitch = rads(pitch);
   matrix[0][0] = 1;
   matrix[0][1] = 0;
   matrix[0][2] = 0;
   matrix[1][0] = 0;
   matrix[1][1] = cos(pitch);
   matrix[1][2] = sin(pitch);
   matrix[2][0] = 0;
   matrix[2][1] = -sin(pitch);
   matrix[2][2] = cos(pitch);
}

void RotMatrix::makeRoll(Float roll) {
   roll = rads(roll);
   matrix[0][0] = cos(roll);
   matrix[0][1] = sin(roll);
   matrix[0][2] = 0;
   matrix[1][0] = -sin(roll);
   matrix[1][1] = cos(roll);
   matrix[1][2] = 0;
   matrix[2][0] = 0;
   matrix[2][1] = 0;
   matrix[2][2] = 1;
}

void RotMatrix::makeIdent() {
   matrix[0][0] = 1;
   matrix[0][1] = 0;
   matrix[0][2] = 0;

   matrix[1][0] = 0;
   matrix[1][1] = 1;
   matrix[1][2] = 0;

   matrix[2][0] = 0;
   matrix[2][1] = 0;
   matrix[2][2] = 1;
}

void RotMatrix::makeProduct(const RotMatrix &a, const RotMatrix &b) {
   assert(this != &a && this != &b);
   matrix[0][0] = 
      a.matrix[0][0] * b.matrix[0][0] +
      a.matrix[0][1] * b.matrix[1][0] +
      a.matrix[0][2] * b.matrix[2][0];
   matrix[0][1] =
      a.matrix[0][0] * b.matrix[0][1] +
      a.matrix[0][1] * b.matrix[1][1] +
      a.matrix[0][2] * b.matrix[2][1];
   matrix[0][2] =
      a.matrix[0][0] * b.matrix[0][2] +
      a.matrix[0][1] * b.matrix[1][2] +
      a.matrix[0][2] * b.matrix[2][2];

   matrix[1][0] = 
      a.matrix[1][0] * b.matrix[0][0] +
      a.matrix[1][1] * b.matrix[1][0] +
      a.matrix[1][2] * b.matrix[2][0];
   matrix[1][1] =
      a.matrix[1][0] * b.matrix[0][1] +
      a.matrix[1][1] * b.matrix[1][1] +
      a.matrix[1][2] * b.matrix[2][1];
   matrix[1][2] =
      a.matrix[1][0] * b.matrix[0][2] +
      a.matrix[1][1] * b.matrix[1][2] +
      a.matrix[1][2] * b.matrix[2][2];

   matrix[2][0] = 
      a.matrix[2][0] * b.matrix[0][0] +
      a.matrix[2][1] * b.matrix[1][0] +
      a.matrix[2][2] * b.matrix[2][0];
   matrix[2][1] =
      a.matrix[2][0] * b.matrix[0][1] +
      a.matrix[2][1] * b.matrix[1][1] +
      a.matrix[2][2] * b.matrix[2][1];
   matrix[2][2] =
      a.matrix[2][0] * b.matrix[0][2] +
      a.matrix[2][1] * b.matrix[1][2] +
      a.matrix[2][2] * b.matrix[2][2];
}

void RotMatrix::makeInverse(const RotMatrix &other) {
   for (int i = 0; i < 3; ++i)
      for (int j = 0; j < 3; ++j)
	 matrix[i][j] = other.matrix[j][i];
}

void RotMatrix::rotate(const Vector &point, Vector &result) const {
   result.x = 
      point.x * matrix[0][0] + 
      point.y * matrix[1][0] +
      point.z * matrix[2][0];
   result.y =
      point.x * matrix[0][1] + 
      point.y * matrix[1][1] +
      point.z * matrix[2][1];
   result.z =
      point.x * matrix[0][2] + 
      point.y * matrix[1][2] +
      point.z * matrix[2][2];
}

void RotMatrix::getGLMatrix(Float mtx[4][4]) const {

   // translate all of the values
   for (int row = 0; row < 3; ++row)
      for (int col = 0; col < 3; ++col)
	 mtx[col][row] = matrix[row][col];

   // set constant 4th row and column values
   mtx[0][3] = 0;
   mtx[1][3] = 0;
   mtx[2][3] = 0;
   mtx[3][0] = 0;
   mtx[3][1] = 0;
   mtx[3][2] = 0;
   mtx[3][3] = 1;
}
