#ifndef ROTMATRIX_H
#define ROTMATRIX_H

#include "Vector.h"

/**
   A Rotational Matrix.  These are used to quickly rotate points around the
   origin.
 */
class RotMatrix {

   Float matrix[3][3];

   public:

      /** Creates an uninitialized rotation matrix. */
      RotMatrix() {}

      /** 
	 Converts the matrix to the rotational matrix for the given yaw
         angle.  Existing contents are ignored.

         @param yaw yaw angle in degrees
       */
      void makeYaw(Float yaw);

      /** 
	 Converts the matrix to the rotational matrix for the given pitch
         angle.  Existing contents are ignored.

         @param pitch pitch angle in degrees
       */
      void makePitch(Float pitch);

      /** 
	 Converts the matrix to the rotational matrix for the given roll
         angle.  Existing contents are ignored.

         @param roll roll angle in degrees
       */
      void makeRoll(Float roll);

      /**
         Converts the matrix to the identity matrix:

	    1 0 0
	    0 1 0
	    0 0 1
       */
      void makeIdent();

      /**
	 Converts the matrix to the product of matrices 'a' and 'b',
	 effectively adding the rotations together.  Existing contents are
	 ignored.

	 'this' must be neither 'a' nor 'b'.
       */
      void makeProduct(const RotMatrix &a, const RotMatrix &b);

      /**
       * Converts the matrix into the inverse of the other matrix.  For
       * rotation matrices, the inverse is always equal to the transpose, so
       * in truth, this method calculates the transpose of "other".
       */
      void makeInverse(const RotMatrix &other);

      /** 
	 Sets 'result' to 'point' rotated around the origin according to the 
	 rotation matrix.
       */
      void rotate(const Vector &point, Vector &result) const;

      /**
	 Fills "matrix" with the OpenGL, 4x4 column major equivalent of the
	 rotation matrix.
       */
      void getGLMatrix(Float matrix[4][4]) const;
};

#endif
