
#include "Texture.h"
#include "PNG.h"
#include <spug/FileReader.h>
#include <spug/Exception.h>

using namespace spug;

Texture::Texture(const char *fileName) : fileName(fileName), textureId(0) {}

Texture::~Texture() {
   if (textureId)
      glDeleteTextures(1, &textureId);
}

namespace {
   // Translation layer from a PNG::Reader to a FileReader
   class ReadTranslater : public PNG::Reader {
      private:
	 FileReader &reader;

      public:
	 ReadTranslater(FileReader &reader) : reader(reader) {}

	 virtual void read(void *buffer, size_t size) {
	    reader.read(reinterpret_cast<Byte *>(buffer), size);
	 }
   };
}

void Texture::load() {
   FileReader reader(fileName.c_str());
   Byte buffer[8];

   // make sure this is a PNG file
   if (reader.read(buffer, sizeof(buffer)) != 8 ||
       !PNG::check(buffer, sizeof(buffer))
       )
      throw Exception(std::string(fileName) + " is not a PNG file");

   // back up to zero
   reader.seek(0);

   // create a PNG object and use it to read the file
   PNG png;
   ReadTranslater localReader(reader);
   png.read(localReader);

   // generate a new texture id and make it the current texture
   glGenTextures(1, &textureId);
   glBindTexture(GL_TEXTURE_2D, textureId);

   // store width & height
   width = png.getWidth();
   height = png.getHeight();

   // set the image data XXX please consolidate open GL funcs
   glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8, width, height, 0, GL_RGB,
		GL_UNSIGNED_BYTE, 
		reinterpret_cast<GLvoid *>(png.getRGB8())
		);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
}

