#ifndef TEXTUREMATERIAL_H
#define TEXTUREMATERIAL_H

#include "Material.h"
#include "Texture.h"
#include "Vector.h"
#include "basictypes.h"

/**
 * Renders a texture on a surface.
 */
class TextureMaterial : public Material {
   private:
      TexturePtr tex;
      int state;

      // scaling factor (multiplied by the units of width & height)
      Float xscale, yscale;

      // calculated bitmap position based on coordinates and scaling factor
      Float xpos, ypos;

      // last recorded coordinate
      Vector lastCoord;

   public:

      TextureMaterial(const TexturePtr &texture, Float xppus, Float yppus) : 
	 tex(texture), 
	 state(0),
	 xscale(xppus / texture->getWidth()),
	 yscale(yppus / texture->getHeight()) {
      }

      virtual void makeCurrent();

      virtual void setCoord(const Vector &coord);
};

SPUG_RCPTR(TextureMaterial);

#endif
