
#include "WurldGtkDriver.h"

#include <X11/Xlib.h>
#include <GL/gl.h>
#include <gdk/gdkx.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include <gtk/gtk.h>
#include <gtk/gtkglwidget.h>
#include <gtk/gtkglinit.h>

#include <spug/FileReader.h>
#include <spug/Tracer.h>

#include "Model.h"
#include "QuadObj.h"
#include "NativePersistor.h"
#include "GameEngine.h"
#include "MouseHandler.h"
#include "Graphics.h"

using namespace spug;


Model cube, map;

int angle = 45;
float depth = 3.0;
#define depthDeltaBase 0.01
float depthDelta = depthDeltaBase;

float near = -20, far = -5;
GtkWidget *glpanel;

void WurldGtkDriver::getDisplayAndWindow(Display *&display, Window &window) {

   // get the X windows display
   GdkDisplay *gdkDisp = gtk_widget_get_display(glpanel);
   display = gdk_x11_display_get_xdisplay(gdkDisp);

   // get the X window object
   window = GDK_WINDOW_XID(glpanel->window);
}

WurldGtkDriver::WurldGtkDriver(GameEngine &engine, MouseHandler &handler) :
   engine(engine),
   mouseHandler(handler),
   initWidth(640),
   initHeight(480) {
}

void WurldGtkDriver::render() {
   glMatrixMode(GL_MODELVIEW);
   glClear(GL_COLOR_BUFFER_BIT);
   glClear(GL_DEPTH_BUFFER_BIT);

#if 0
   // clear the background
   glLoadIdentity();
   glTranslatef(0, 0, 100);
   glColor3f(0, 0, 0);
   glBegin(GL_POLYGON);
      glVertex2f(8, 8);
      glVertex2f(8, -8);
      glVertex2f(-8, -8);
      glVertex2f(-8, 8);
   glEnd();
#endif

   engine.handleIdle();
}

void WurldGtkDriver::initialize() {
   // XXX move the rest of "configure" here
   engine.loadTextures();
}

void WurldGtkDriver::shutdown() {
   mouseHandler.deactivate();
   gtk_main_quit();
}

void WurldGtkDriver::handleKeyPress(int keyCode) {
   switch (keyCode) {
      case 'q':
         shutdown();
	 break;
      case 'g':
	 // toggle mouse handler activation
	 if (mouseHandler.isActive())
	    mouseHandler.deactivate();
	 else
	    mouseHandler.activate(this);
	 break;
      case 'd':
	 engine.setForward(true);
	 break;
      case 'z':
	 engine.setLeft(true);
	 break;
      case 'c':
	 engine.setRight(true);
	 break;
      case 'x':
	 engine.setBackward(true);
	 break;
   }
}

void WurldGtkDriver::handleKeyRelease(int keyCode) {
   switch (keyCode) {
      case 'd':
	 engine.setForward(false);
	 break;
      case 'x':
	 engine.setBackward(false);
	 break;
      case 'z':
	 engine.setLeft(false);
	 break;
      case 'c':
	 engine.setRight(false);
	 break;
   }
}

extern "C" {

static void expose(GtkWidget *widget, GdkEventExpose *event, gpointer data) {
   GdkGLContext *glcontext = gtk_widget_get_gl_context(widget);
   GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable(widget);

   if (!gdk_gl_drawable_gl_begin(gldrawable, glcontext))
      return;

   reinterpret_cast<WurldGtkDriver *>(data)->render();

   if (gdk_gl_drawable_is_double_buffered (gldrawable))
      gdk_gl_drawable_swap_buffers (gldrawable);
   else
      glFlush ();

   gdk_gl_drawable_gl_end(gldrawable);
}

#if 0
void reshape(int width, int height) {

   // Set the new viewport size
   glViewport(0, 0, (GLint)width, (GLint)height);

   // Clear the window
   glClear(GL_COLOR_BUFFER_BIT);
}
#endif


static void realize(GtkWidget *widget, gpointer *data) {
   GdkGLContext *glcontext = gtk_widget_get_gl_context(widget);
   GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable(widget);

   std::cout << "Window realized" << std::endl;
   //mouseHandler.activate(widget);
   if (!gdk_gl_drawable_gl_begin(gldrawable, glcontext))
      return;

#if 0
   // Set the drawing color
   glColor3f(1.0, 0, 0);

   // Specify which primitive type is to be drawn
   glBegin(GL_POLYGON);
	  // Specify verticies in quad
	  glVertex2f(-0.5, -0.5);
	  glVertex2f(-0.5, 0.5);
	  glVertex2f(0.5, 0.5);
	  glVertex2f(0.5, -0.5);
   glEnd();
#endif
   reinterpret_cast<WurldGtkDriver *>(data)->render();

   if (gdk_gl_drawable_is_double_buffered (gldrawable))
      gdk_gl_drawable_swap_buffers (gldrawable);
   else
      glFlush ();

   gdk_gl_drawable_gl_end(gldrawable);
}

static gboolean configure(GtkWidget *widget, GdkEventConfigure *event, 
                          gpointer data
                          ) {
   GdkGLContext *glcontext = gtk_widget_get_gl_context(widget);
   GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable(widget);

   if (!gdk_gl_drawable_gl_begin(gldrawable, glcontext))
      return FALSE;

   Graphics::getInstance().initialize(widget->allocation.width, 
		         	      widget->allocation.height
		         	      );
   gdk_gl_drawable_gl_end(gldrawable);

   reinterpret_cast<WurldGtkDriver *>(data)->initialize();
#if 0
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glFrustum(-1, 1, -1, 1, 1, 100);
   glMatrixMode(GL_MODELVIEW);
#endif


   return TRUE;
}

static gboolean idle(GtkWidget *widget) {
   /* Invalidate the whole window. */
   gdk_window_invalidate_rect(widget->window, &widget->allocation, FALSE);

   /* Update synchronously. */
   gdk_window_process_updates(widget->window, FALSE);

   return TRUE;
}

static void MainWin_destroy(GtkWidget *window, gpointer data);

static gboolean mouseMove(GtkWidget *widget, GdkEventMotion *event, 
                          WurldGtkDriver *driver) {
   driver->handleMouseMove(static_cast<int>(event->x), 
                           static_cast<int>(event->y)
                           );
}

static gboolean keyPress(GtkWidget *widget, GdkEventKey *event, 
                         WurldGtkDriver *driver) {
   driver->handleKeyPress(event->keyval);
   return TRUE;
}

static gboolean keyRelease(GtkWidget *widget, GdkEventKey *event, 
                           WurldGtkDriver *driver) {
   driver->handleKeyRelease(event->keyval);
   return TRUE;
}

void MainWin_destroy(GtkWidget *window, gpointer data) {
   reinterpret_cast<WurldGtkDriver *>(data)->shutdown();
}

}; // extern "C"

int WurldGtkDriver::main(int argc, const char **argv) {


   GtkWidget *vbox;
   GdkGLConfig *glconfig;

   gtk_init(&argc, const_cast<char ***>(&argv));
   gtk_gl_init(&argc, const_cast<char ***>(&argv));

   if (argc > 1)
      near = atof(argv[1]);
   if (argc > 2)
      far = atof(argv[2]);

   // create a cube
   engine.makeCube(cube);

   // enable detailed tracing of the persistor
   Tracer::get().enable(Tracer::get().getLevel("NativePersistor.detailed"));

   mainWin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_signal_connect(GTK_OBJECT(mainWin), "destroy", 
		      G_CALLBACK(MainWin_destroy),
		      this
		      );

   /* Try double-buffered visual */
   glconfig = gdk_gl_config_new_by_mode((GdkGLConfigMode)(GDK_GL_MODE_RGB |
					 GDK_GL_MODE_DEPTH |
					 GDK_GL_MODE_DOUBLE
					 )
					);
   if (glconfig == NULL) {
      g_print("*** Cannot find the double-buffered visual.\n");
      g_print("*** Trying single-buffered visual.\n");

      /* Try single-buffered visual */
      glconfig = gdk_gl_config_new_by_mode((GdkGLConfigMode)(GDK_GL_MODE_RGB |
					   GDK_GL_MODE_DEPTH)
					   );
      if (glconfig == NULL) {
	 g_print("*** No appropriate OpenGL-capable visual found.\n");
	 exit(1);
      }
   }

   //examine_gl_config_attrib (glconfig);

   vbox = gtk_vbox_new(FALSE, 0);
   gtk_container_add(GTK_CONTAINER(mainWin), vbox);
   gtk_widget_show(vbox);

   glpanel = gtk_drawing_area_new();
   gtk_widget_set_size_request(glpanel, initWidth, initHeight);
   mouseHandler.setWindowSize(initWidth, initHeight);

   /* Set OpenGL-capability to the widget. */
   gtk_widget_set_gl_capability(glpanel, glconfig, NULL, TRUE,
			        GDK_GL_RGBA_TYPE
			        );

   /* add signal handlers */
   g_signal_connect_after(G_OBJECT(glpanel), "realize",
                          G_CALLBACK(realize), 
			  this
			  );
   g_signal_connect(G_OBJECT(glpanel), "configure_event",
		    G_CALLBACK(configure), 
		    this
		    );
   g_signal_connect(G_OBJECT (glpanel), "expose_event",
		    G_CALLBACK(expose), 
		    this
		    );
   gtk_signal_connect(GTK_OBJECT(glpanel), "motion_notify_event",
		      G_CALLBACK(mouseMove),
		      this
		      );
   gtk_signal_connect(GTK_OBJECT(mainWin), "key_press_event",
		      G_CALLBACK(keyPress),
		      this
		      );
   gtk_signal_connect(GTK_OBJECT(mainWin), "key_release_event",
		      G_CALLBACK(keyRelease),
		      this
		      );

   // get mouse and expose events in the GL panel
   gtk_widget_set_events(GTK_WIDGET(glpanel), 
			 GDK_EXPOSURE_MASK | GDK_LEAVE_NOTIFY_MASK | 
			  GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | 
			  GDK_POINTER_MOTION_MASK
		         );	

   // when idle, repaint
   g_idle_add_full(GDK_PRIORITY_REDRAW, (GSourceFunc)idle, glpanel, NULL);

   // pack and show the GL panel
   gtk_box_pack_start(GTK_BOX(vbox), glpanel, TRUE, TRUE, 0);
   gtk_widget_show(glpanel);

   gtk_widget_show(mainWin);
   gtk_main();
}

void WurldGtkDriver::setDisplaySize(int width, int height) {
   initWidth = width;
   initHeight = height;
}

DisplayDriver::Size WurldGtkDriver::getSize() const {
   return Size(glpanel->allocation.width, glpanel->allocation.height);
}

void WurldGtkDriver::hideMouse() {
}

/** warps the mouse to the center of the display */
void WurldGtkDriver::warpMouse(int x, int y) {
   Display *display;
   Window win;
   getDisplayAndWindow(display, win);

   XWarpPointer(display, None, win, 0, 0, 0, 0, x, y);
}

void WurldGtkDriver::grabMouse(bool grab) {
   // grab the mouse
   Display *display;
   Window window = 0;
   getDisplayAndWindow(display, window);
   if (grab) {
      int rc = XGrabPointer(display, window, False,
                            PointerMotionMask | ButtonPressMask |
                             ButtonReleaseMask | PointerMotionMask |
                             Button1MotionMask | Button2MotionMask |
                             Button3MotionMask | ButtonMotionMask | Button1Mask |
                             Button2Mask | Button3Mask,
                             GrabModeAsync,
                             GrabModeAsync,
                             window,
                             None,
                             CurrentTime
                             );
      if (rc != GrabSuccess) {
         char buffer[256];
         XGetErrorText(display, rc, buffer, sizeof(buffer));
         std::cout << "grab failed" << std::endl;
         std::cout << buffer << std::endl;
      } else {
         std::cout << "grabbed and centered" << std::endl;
      }
   } else {
      XUngrabPointer(display, CurrentTime);
   }
}

