#ifndef GTKDRIVER_H
#define GTKDRIVER_H

#include "DisplayDriver.h"
#include "GameEngine.h"
#include "MouseHandler.h"

class WurldGtkDriver : public DisplayDriver {
   private:
      GameEngine &engine;
      MouseHandler &mouseHandler;
      GtkWidget *mainWin;
      int initWidth, initHeight;

   protected:
      void getDisplayAndWindow(Display *&display, Window &window);

   public:
      WurldGtkDriver(GameEngine &engine, MouseHandler &mouseHandler);

      void initialize();

      // shuts down the application
      void shutdown();

      // called to render the scene
      void render();

      // called when the mouse is moved
      void handleMouseMove(int x, int y) {
         mouseHandler.handleMouseMove(x, y);
      }

      // called on a keypress
      void handleKeyPress(int code);

      // called on a key release
      void handleKeyRelease(int code);

      virtual int main(int argc, const char **argv);

      virtual void setDisplaySize(int width, int height);

      Size getSize() const;

      void hideMouse();

      void warpMouse(int x, int y);

      void grabMouse(bool grab);
};

#endif
