#ifndef BASICTYPES_H
#define BASICTYPES_H

#include <math.h>
#include <GL/gl.h>

typedef double Float;

// muahaha
namespace {
   const Float infinity = 1e+72;
};

template <class T>
T rads(T val) { return (val / 180) * M_PI; }

template <class T>
T degrees(T val) { return (val / M_PI) * 180; }

/**
 * Templatized absolute value function.
 */
template <class T>
T mabs(T val);

template <>
inline float mabs(float val) { return fabsf(val); }

template <>
inline double mabs(double val) { return fabs(val); }

/**
 * Given an angle in degrees, returns the equivalent angle betweem -180 and
 * 180 degrees.
 */
template <class T>
inline T normalDegrees(T angle) {
   T a = mabs<T>(angle);
   while (a > 180) {
      angle -= (angle / a) * 360;
      a = mabs<T>(angle);
   }
   return angle;
}

/**
 * Templatized check for floating point zero
 */
template <class T>
bool isZero(T val);

template <>
inline bool isZero(float val) {
   return val < 1e-72 && val > -1e-72;
}

template <>
inline bool isZero(double val) {
   return val < 1e-144 && val > -1e-144;
}

template <class T>
void glColor(T red, T green, T blue);

template <>
inline void glColor<float>(float red, float green, float blue) {
   glColor3f(red, green, blue);
}

template <>
inline void glColor<double>(double red, double green, double blue) {
   glColor3d(red, green, blue);
}

template <class T>
void glColor(T red, T green, T blue, T alpha);

template <>
inline void glColor<float>(float red, float green, float blue, float alpha) {
   glColor4f(red, green, blue, alpha);
}

template <>
inline void glColor<double>(double red, double green, double blue, 
			    double alpha
			    ) {
   glColor4d(red, green, blue, alpha);
}

template <class T>
void glVertex(T x, T y, T z);

template <>
inline void glVertex<float>(float x, float y, float z) {
   glVertex3f(x, y, z);
}

template <>
inline void glVertex<double>(double x, double y, double z) {
   glVertex3d(x, y, z);
}

template <class T>
void glTranslate(T x, T y, T z);

template <>
inline void glTranslate<float>(float x, float y, float z) {
   glTranslatef(x, y, z);
}

template <>
inline void glTranslate<double>(double x, double y, double z) {
   glTranslated(x, y, z);
}

template <class T>
void glRotate(T angle, T x, T y, T z);

template <>
inline void glRotate<float>(float angle, float x, float y, float z) {
   glRotatef(angle, x, y, z);
}

template <>
inline void glRotate<double>(double angle, double x, double y, double z) {
   glRotated(angle, x, y, z);
}

template <class T>
void glMultMatrix(const T m[4][4]);

template <>
inline void glMultMatrix<float>(const float m[4][4]) {
   glMultMatrixf(reinterpret_cast<const float *>(m));
}

template <>
inline void glMultMatrix<double>(const double m[4][4]) {
   glMultMatrixd(reinterpret_cast<const double *>(m));
}

template <class T>
void glTexCoord(T x, T y);

template <>
inline void glTexCoord<float>(float x, float y) {
   glTexCoord2f(x, y);
}

template <>
inline void glTexCoord<double>(double x, double y) {
   glTexCoord2f(x, y);
}

template <class T>
void glNormal(T x, T y, T z);

template <>
inline void glNormal(float x, float y, float z) {
   glNormal3d(x, y, z);
}

template <>
inline void glNormal(double x, double y, double z) {
   glNormal3f(x, y, z);
}

#endif
