// OpenGL Tutorial
// Hello_World.c

/*************************************************************************
This program essentially opens a window, clears it, sets the drawing
color, draws the object, and flushes the drawing buffer.  It then goes
into an infinite loop accepting events and calling the appropriate
functions.
*************************************************************************/

// gcc -o Hello_World  Hello_World.c -lX11 -lMesaGL -lMesaGLU -lMesatk -lm

#include <GL/gl.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include <gtk/gtk.h>
#include <gtk/gtkglwidget.h>

void expose(GtkWidget *widget, GdkEventExpose *event, gpointer data) {
   GdkGLContext *glcontext = gtk_widget_get_gl_context(widget);
   GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable(widget);

   if (!gdk_gl_drawable_gl_begin(gldrawable, glcontext))
      return;

   render();

   if (gdk_gl_drawable_is_double_buffered (gldrawable))
      gdk_gl_drawable_swap_buffers (gldrawable);
   else
      glFlush ();

   gdk_gl_drawable_gl_end(gldrawable);
}

#if 0
void reshape(int width, int height) {

   // Set the new viewport size
   glViewport(0, 0, (GLint)width, (GLint)height);

   // Clear the window
   glClear(GL_COLOR_BUFFER_BIT);
}
#endif

int angle = 45;
float depth = 3.0;
#define depthDeltaBase 0.01
float depthDelta = depthDeltaBase;
float near = 1.5, far = 7.5;

void render() {
   glMatrixMode(GL_MODELVIEW);
   glClear(GL_COLOR_BUFFER_BIT);
   glClear(GL_DEPTH_BUFFER_BIT);

   // clear the background
   glLoadIdentity();
   glTranslatef(0, 0, 8);
   glColor3f(0, 0, 0);
   glBegin(GL_POLYGON);
      glVertex2f(8, 8);
      glVertex2f(8, -8);
      glVertex2f(-8, -8);
      glVertex2f(-8, 8);
   glEnd();

   glLoadIdentity();
   glTranslatef(0, 0, depth);
   glRotatef(angle, 1, 1, 1);
   angle = (angle + 1) % 360;
   depth += depthDelta;
   if (depth < near)
      depthDelta = depthDeltaBase;
   else if (depth > far)
      depthDelta = -depthDeltaBase;

   // red square up front
   glColor3f(1, 0, 0);
   glBegin(GL_POLYGON);
      glVertex3f(1, 1, 1);
      glVertex3f(-1, 1, 1);
      glVertex3f(-1, -1, 1);
      glVertex3f(1, -1, 1);
   glEnd();

   // green square right
   glColor3f(0, 1, 0);
   glBegin(GL_POLYGON);
      glVertex3f(1, 1, 1);
      glVertex3f(1, 1, 2);
      glVertex3f(1, -1, 2);
      glVertex3f(1, -1, 1);
   glEnd();

   // blue square bottom
   glColor3f(0, 0, 1);
   glBegin(GL_POLYGON);
      glVertex3f(1, -1, 1);
      glVertex3f(1, -1, 2);
      glVertex3f(-1, -1, 2);
      glVertex3f(-1, -1, 1);
   glEnd();

   // yellow square top
   glColor3f(1, 1, 0);
   glBegin(GL_POLYGON);
      glVertex3f(1, 1, 1);
      glVertex3f(1, 1, 2);
      glVertex3f(-1, 1, 2);
      glVertex3f(-1, 1, 1);
   glEnd();

   // purple square back
   glColor3f(1, 0, 1);
   glBegin(GL_POLYGON);
      glVertex3f(1, 1, 2);
      glVertex3f(-1, 1, 2);
      glVertex3f(-1, -1, 2);
      glVertex3f(1, -1, 2);
   glEnd();

   // cyan square left
   glColor3f(0, 1, 1);
   glBegin(GL_POLYGON);
      glVertex3f(-1, 1, 1);
      glVertex3f(-1, 1, 2);
      glVertex3f(-1, -1, 2);
      glVertex3f(-1, -1, 1);
   glEnd();


}

void realize(GtkWidget *widget, gpointer *data) {
   GdkGLContext *glcontext = gtk_widget_get_gl_context(widget);
   GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable(widget);

   if (!gdk_gl_drawable_gl_begin(gldrawable, glcontext))
      return;

#if 0
   // Set the drawing color
   glColor3f(1.0, 0, 0);

   // Specify which primitive type is to be drawn
   glBegin(GL_POLYGON);
	  // Specify verticies in quad
	  glVertex2f(-0.5, -0.5);
	  glVertex2f(-0.5, 0.5);
	  glVertex2f(0.5, 0.5);
	  glVertex2f(0.5, -0.5);
   glEnd();
#endif
   render();

   if (gdk_gl_drawable_is_double_buffered (gldrawable))
      gdk_gl_drawable_swap_buffers (gldrawable);
   else
      glFlush ();

   gdk_gl_drawable_gl_end(gldrawable);
}

gboolean configure(GtkWidget *widget, GdkEventConfigure *event, 
		   gpointer data
		   ) {
   GdkGLContext *glcontext = gtk_widget_get_gl_context(widget);
   GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable(widget);

   if (!gdk_gl_drawable_gl_begin(gldrawable, glcontext))
      return;

   glViewport(0, 0, widget->allocation.width, widget->allocation.height);
   gdk_gl_drawable_gl_end(gldrawable);

   glClearColor(0, 0, 0, 0);
   glShadeModel(GL_FLAT);
   glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
   glEnable(GL_DEPTH_TEST);

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glFrustum(-8, 8, -8, 8, 10, 100);

   glMatrixMode(GL_MODELVIEW);

   return TRUE;
}

static gboolean idle(GtkWidget *widget) {
   /* Invalidate the whole window. */
   gdk_window_invalidate_rect(widget->window, &widget->allocation, FALSE);

   /* Update synchronously. */
   gdk_window_process_updates(widget->window, FALSE);

   return TRUE;
}

void MainWin_destroy(GtkWidget *window, gpointer data) {
   gtk_main_quit();
}

void main(int argc, char **argv) {

#if 0
   // Open a window, name it "Hello World"
   if (tkInitWindow("Hello World") == GL_FALSE) {
      tkQuit();
   }

   // Set the clear color to black
   glClearColor(0.0, 0.0, 0.0, 0.0);

   // Assign expose() to be the function called whenever
   // an expose event occurs
   tkExposeFunc(expose);

   // Assign reshape() to be the function called whenever 
   // a reshape event occurs
   tkReshapeFunc(reshape);

   // Assign draw() to be the function called whenever a display
   // event occurs, generally after a resize or expose event
   tkDisplayFunc(draw);

   // Pass program control to tk's event handling code
   // In other words, loop forever
   tkExec();
#endif

   GtkWidget *mainWin, *vbox, *glpanel;
   GdkGLConfig *glconfig;

   gtk_init(&argc, &argv);
   gtk_gl_init(&argc, &argv);

   if (argc > 1)
      near = atof(argv[1]);
   if (argc > 2)
      far = atof(argv[2]);

   mainWin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_signal_connect(G_OBJECT(mainWin), "destroy", 
		      G_CALLBACK(MainWin_destroy),
		      NULL
		      );

   /* Try double-buffered visual */
   glconfig = gdk_gl_config_new_by_mode(GDK_GL_MODE_RGB |
					GDK_GL_MODE_DEPTH |
					GDK_GL_MODE_DOUBLE
					);
   if (glconfig == NULL) {
      g_print("*** Cannot find the double-buffered visual.\n");
      g_print("*** Trying single-buffered visual.\n");

      /* Try single-buffered visual */
      glconfig = gdk_gl_config_new_by_mode(GDK_GL_MODE_RGB |
					   GDK_GL_MODE_DEPTH
					   );
      if (glconfig == NULL) {
	 g_print("*** No appropriate OpenGL-capable visual found.\n");
	 exit(1);
      }
   }

   //examine_gl_config_attrib (glconfig);

   vbox = gtk_vbox_new(FALSE, 0);
   gtk_container_add(GTK_CONTAINER(mainWin), vbox);
   gtk_widget_show(vbox);

   glpanel = gtk_drawing_area_new();
   gtk_widget_set_size_request(glpanel, 640, 480);

   /* Set OpenGL-capability to the widget. */
   gtk_widget_set_gl_capability(glpanel, glconfig, NULL, TRUE,
			        GDK_GL_RGBA_TYPE
			        );

   /* add signal handlers */
   g_signal_connect_after(G_OBJECT(glpanel), "realize",
                          G_CALLBACK(realize), 
			  NULL
			  );
   g_signal_connect(G_OBJECT(glpanel), "configure_event",
		    G_CALLBACK(configure), 
		    NULL
		    );
   g_signal_connect(G_OBJECT (glpanel), "expose_event",
		    G_CALLBACK(expose), 
		    NULL
		    );

   // when idle, repaint
   g_idle_add_full(GDK_PRIORITY_REDRAW, (GSourceFunc)idle, glpanel, NULL);

   // pack and show the GL panel
   gtk_box_pack_start(GTK_BOX(vbox), glpanel, TRUE, TRUE, 0);
   gtk_widget_show(glpanel);

   gtk_widget_show(mainWin);
   gtk_main();
}

