#==============================================================================
#
#  $Id$
#
"""
   Some basic utilities.
"""
#
#  Copyright (C) 2007 Michael A. Muller
#
#  Permission is granted to use, modify and redistribute this code,
#  providing that the following conditions are met:
#
#  1) This copyright/licensing notice must remain intact.
#  2) If the code is modified and redistributed, the modifications must 
#  include documentation indicating that the code has been modified.
#  3) The author(s) of this code must be indemnified and held harmless
#  against any damage resulting from the use of this code.
#
#  This code comes with ABSOLUTELY NO WARRANTEE, not even the implied 
#  warrantee of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#
#==============================================================================

import wurld

def vecToStr(vec):
   return '(%g, %g, %g)' % (vec.x, vec.y, vec.z)

def showModel(model):
   """
      Wraps the model in a moveable and a python object and adds it to the
      renderables.
   """
   moveable = wurld.createMoveable()
   moveable.setModel(model)
   obj = wurld.createPythonObject()
   obj.setMoveable(moveable)
   wurld.getGameEngine().addRenderableObject(obj)
   return obj

class Rot:
   """
      A class to make dealing with rotation matrices a little less verbose.
   """

   def __init__(self, rotMatrix):
      self.mtx = rotMatrix

   @staticmethod
   def yaw(deg):
      return Rot(wurld.createYawRotMatrix(deg))

   @staticmethod
   def pitch(deg):
      return Rot(wurld.createPitchRotMatrix(deg))

   @staticmethod
   def roll(deg):
      return Rot(wurld.createRollRotMatrix(deg))

   def __mul__(self, other):
      temp = Rot.yaw(0)
      temp.mtx.makeProduct(self.mtx, other.mtx)
      return temp

   def rotate(self, vector):
      temp = wurld.createVector(0, 0, 0)
      self.mtx.rotate(vector, temp)
      return temp

